import os
import gevent
import gevent.subprocess
import sys
import json
from base64 import b64decode
sys.path.insert(0, '/motadata/motadata/motadata python engine')
from com.motadata.commons.sshclient import TraceOrgSSHClient
from com.motadata.util.loggerutil import TraceOrgLogger
_logger = TraceOrgLogger("Script Metric Collector")


def convert_to_str(value):
    result = 0

    try:
        if value:

            result = str(value).strip().replace("\r", "")

    except Exception:

        _logger.error()

    return result

def convert_to_int(value):
    result = 0

    try:
        if value:

            result = int(round(float(str(value).replace("\r", "").strip())))

    except Exception:

        _logger.error()

    return result


def get_ssh_client(plugin_context, host):
    ssh_client = None

    try:

        port = 22

        user_name = convert_to_str(plugin_context.get('username'))

        password = convert_to_str(plugin_context.get('password'))

        if plugin_context.get('port') and len(convert_to_str(plugin_context.get('port'))) > 0:

            port = convert_to_int(plugin_context.get('port'))

        timeout = convert_to_int(plugin_context['timeout'])

        if plugin_context.get('private-key-name') and len(convert_to_str(plugin_context.get('private-key-name'))) > 0:

            private_key = convert_to_str(plugin_context.get('private-key-name'))

            passphrase = None

            if plugin_context.get('pass-phrase') and len(convert_to_str(plugin_context.get('pass-phrase'))) > 0:

                passphrase = convert_to_str(plugin_context.get('pass-phrase'))

            ssh_client = TraceOrgSSHClient(host=host,
                                           port=port,
                                           username=user_name,
                                           password=password,
                                           timeout=timeout,
                                           key_file=private_key,
                                           passphrase=passphrase)

        else:

            ssh_client = TraceOrgSSHClient(host=host,
                                           port=port,
                                           username=user_name,
                                           password=password,
                                           timeout=timeout)


    except Exception:
        _logger.error()

    return ssh_client

def collect(context):

    metrics = {}

    _logger.info("ssh context " + str(context))

    if context and context.get('plugin-context'):

        plugin_context = context.get("plugin-context")

        ssh_client = None

        host = None

        timeout = 60

        if context.get("metric-collect-timeout"):

            timeout = convert_to_int(context.get("metric-collect-timeout"))

        plugin_context['timeout'] = timeout

        try:

            if plugin_context.get('username') and plugin_context.get('host'):

                _logger.debug("metric collection request:" + plugin_context.get("host"))

                if plugin_context.get("script-type") is not None and str(plugin_context.get("script-type")).lower() == 'script':

                    command = "sh " + convert_to_str(plugin_context.get("content"))

                elif plugin_context.get("script-type") is not None and str(plugin_context.get("script-type")).lower() == 'command':

                    command = convert_to_str(plugin_context.get("content"))

                    _logger.info("ssh command")

                host = convert_to_str(plugin_context.get('host'))

                ssh_client = get_ssh_client(plugin_context, host)

                if ssh_client:

                    if ssh_client.init():

                        output = ssh_client.execute_command(command).strip()

                        metrics['result'] = output

                    else:
                        metrics['error-code'] = 'Invalid Credentials {}'.format(host)
                else:
                    metrics['error-code'] = 'No Valid Credentials found {}'.format(host)
            else:
                metrics['error-code'] = 'No valid credentials found {}'.format(host)

        except Exception as exception:

            _logger.warn("failed to collect linux metric " + host)

            metrics['error-code'] = exception.__class__.__name__

        finally:
            if ssh_client:
                ssh_client.destroy()

    return metrics




def main():
    if len(sys.argv) > 0 and sys.argv[1] == 'collector':

        context = {}


        try:
            context = json.loads(b64decode(sys.argv[2]))

            contextUpdated = {}

            contextUpdated.update({'plugin-context':context})

            ssh_client = get_ssh_client(context, context['host'])

            if ssh_client and ssh_client.init():

                result = collect(contextUpdated)

                if result is not None and len(result) > 0:

                    context['result'] = result
                else:

                    if ssh_client.exception is not None:

                        context['error-code'] = convert_to_str(ssh_client.exception)

                    else:
                        context['error-code'] = "Failed to discover "+context["host"]

                ssh_client.destroy()
            else:
                context['error-code'] = ssh_client.exception.__class__.__name__

            del context['password']

            del context['username']

        except Exception as exception:

            context['error-code'] = exception.__class__.__name__

            _logger.error()
        finally:
            print (json.dumps(context))

if __name__ == "__main__":
    main()
